#include <Audio.h>
#include <Wire.h>


#define CHORUS_DELAY_LENGTH_SAMPLES (15*AUDIO_BLOCK_SAMPLES) // one AUDIO_BLOCK_SAMPLES = 2.902ms; you need doubled length, e.g. delay point is 20ms, so you need up to 40ms delay!

#include "effect_modulated_delay.h"

short delayline[CHORUS_DELAY_LENGTH_SAMPLES];

AudioSynthWaveform        testtone;
AudioEffectModulatedDelay modchorus;
AudioSynthWaveform        modulator;
AudioMixer4               mixer;
AudioOutputI2S            audioOutput;

// Create Audio connections between the components
// Both channels of the audio input go to the chorus effect
AudioConnection c1(testtone, 0, modchorus, 0);
AudioConnection c2(modulator, 0, modchorus, 1);
AudioConnection c3(testtone, 0, mixer, 0);
AudioConnection c4(modchorus, 0, mixer, 1);
AudioConnection c5(mixer, 0, audioOutput, 0);
AudioConnection c6(mixer, 0, audioOutput, 1);
AudioControlSGTL5000 audioShield;

void setup() {

  Serial.begin(38400);
  delay(180);

  AudioMemory(20);

  memset(delayline, 0, sizeof(int16_t)*CHORUS_DELAY_LENGTH_SAMPLES);
  if (!modchorus.begin(delayline, CHORUS_DELAY_LENGTH_SAMPLES)) {
    Serial.println("AudioEffectChorus - begin failed");
    while (1);
  }

  audioShield.enable();
  audioShield.volume(1.0);
  audioShield.dacVolumeRamp();
  audioShield.dacVolume(1.0);
  audioShield.unmuteHeadphone();
  audioShield.volume(0.5, 0.5); // Headphone volume
  audioShield.unmuteLineout();
  audioShield.lineOutLevel(29);

  testtone.begin(WAVEFORM_SINE);
  testtone.phase(0);
  testtone.offset(0.0);
  testtone.amplitude(0.5);
  testtone.frequency(1000);

  modulator.begin(WAVEFORM_SINE);
  modulator.phase(0);
  modulator.offset(0.0);
  modulator.amplitude(0.5);
  modulator.frequency(0.5);

  mixer.gain(0, 0.0);
  mixer.gain(1, 0.5);

  Serial.println("setup done");
  AudioProcessorUsageMaxReset();
  AudioMemoryUsageMaxReset();
}

elapsedMillis last_time = 0;

void loop()
{
  if (last_time >= 1000) {
    Serial.print("Proc = ");
    Serial.print(AudioProcessorUsage());
    Serial.print(" (");
    Serial.print(AudioProcessorUsageMax());
    Serial.print("),  Mem = ");
    Serial.print(AudioMemoryUsage());
    Serial.print(" (");
    Serial.print(AudioMemoryUsageMax());
    Serial.println(")");
    last_time = 0;
  }
}
